@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -14,8 +13,10 @@
 
 from functools import wraps
 
+import six
 from webob import exc
 
+from heat.common.i18n import _
 from heat.common import identifier
 
 
@@ -85,7 +86,7 @@ def get_allowed_params(params, whitelist):
     '''
     allowed_params = {}
 
-    for key, get_type in whitelist.iteritems():
+    for key, get_type in six.iteritems(whitelist):
         value = None
         if get_type == 'single':
             value = params.get(key)
