@@ -1,3 +1,4 @@
+#
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,11 +15,11 @@
 # limitations under the License.
 
 from oslo.config import cfg
+from oslo.utils import importutils
 from webob.exc import HTTPBadRequest
 from webob.exc import HTTPUnauthorized
 
 from heat.common import wsgi
-from heat.openstack.common import importutils
 
 
 class AuthUrlFilter(wsgi.Middleware):
@@ -33,7 +34,7 @@ class AuthUrlFilter(wsgi.Middleware):
             return self.conf['auth_uri']
         else:
             # Import auth_token to have keystone_authtoken settings setup.
-            auth_token_module = 'keystoneclient.middleware.auth_token'
+            auth_token_module = 'keystonemiddleware.auth_token'
             importutils.import_module(auth_token_module)
             return cfg.CONF.keystone_authtoken.auth_uri
 
