@@ -1,5 +1,3 @@
-
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -16,7 +14,6 @@
 """
 Routines for configuring Heat
 """
-import copy
 import logging as sys_logging
 import os
 
@@ -24,11 +21,7 @@ from eventlet.green import socket
 from oslo.config import cfg
 
 from heat.common import wsgi
-
 from heat.openstack.common import log as logging
-from heat.openstack.common import rpc
-
-logger = logging.getLogger(__name__)
 
 paste_deploy_group = cfg.OptGroup('paste_deploy')
 paste_deploy_opts = [
@@ -59,25 +52,35 @@ service_opts = [
                help='Instance connection to CFN/CW API validate certs if SSL '
                     'is used.'),
     cfg.StrOpt('region_name_for_services',
-               default=None,
                help='Default region name used to get services endpoints.'),
     cfg.StrOpt('heat_stack_user_role',
                default="heat_stack_user",
                help='Keystone role for heat template-defined users.'),
-    cfg.StrOpt('stack_user_domain',
+    cfg.StrOpt('stack_user_domain_id',
+               deprecated_opts=[cfg.DeprecatedOpt('stack_user_domain',
+                                                  group=None)],
                help='Keystone domain ID which contains heat template-defined '
-                    'users.'),
+                    'users. If this option is set, stack_user_domain_name '
+                    'option will be ignored.'),
+    cfg.StrOpt('stack_user_domain_name',
+               help='Keystone domain name which contains heat '
+                    'template-defined users. If `stack_user_domain_id` option '
+                    'is set, this option is ignored.'),
     cfg.StrOpt('stack_domain_admin',
                help='Keystone username, a user with roles sufficient to '
                     'manage users and projects in the stack_user_domain.'),
     cfg.StrOpt('stack_domain_admin_password',
+               secret=True,
                help='Keystone password for stack_domain_admin user.'),
     cfg.IntOpt('max_template_size',
                default=524288,
                help='Maximum raw byte size of any template.'),
     cfg.IntOpt('max_nested_stack_depth',
                default=3,
-               help='Maximum depth allowed when using nested stacks.')]
+               help='Maximum depth allowed when using nested stacks.'),
+    cfg.IntOpt('num_engine_workers',
+               default=1,
+               help='Number of heat-engine processes to fork and run.')]
 
 engine_opts = [
     cfg.StrOpt('instance_user',
@@ -103,7 +106,10 @@ engine_opts = [
                       'stored password or trusts.')),
     cfg.ListOpt('trusts_delegated_roles',
                 default=['heat_stack_owner'],
-                help=_('Subset of trustor roles to be delegated to heat.')),
+                help=_('Subset of trustor roles to be delegated to heat.'
+                       ' If trusts_delegated_roles is set to [],'
+                       ' all roles of a user will be delegated to heat'
+                       ' when creating a stack.')),
     cfg.IntOpt('max_resources_per_stack',
                default=1000,
                help='Maximum resources allowed per top-level stack.'),
@@ -111,6 +117,11 @@ engine_opts = [
                default=100,
                help=_('Maximum number of stacks any one tenant may have'
                       ' active at one time.')),
+    cfg.IntOpt('action_retry_limit',
+               default=5,
+               help=_('Number of times to retry to bring a '
+                      'resource to a non-error state. Set to 0 to disable '
+                      'retries.')),
     cfg.IntOpt('event_purge_batch_size',
                default=10,
                help=_('Controls how many events will be pruned whenever a '
@@ -130,13 +141,17 @@ engine_opts = [
                default=2,
                help=_('RPC timeout for the engine liveness check that is used'
                       ' for stack locking.')),
+    cfg.BoolOpt('enable_cloud_watch_lite',
+                default=True,
+                help=_('Enable the legacy OS::Heat::CWLiteAlarm resource.')),
+    cfg.BoolOpt('enable_stack_abandon',
+                default=False,
+                help=_('Enable the preview Stack Abandon feature.')),
+    cfg.BoolOpt('enable_stack_adopt',
+                default=False,
+                help=_('Enable the preview Stack Adopt feature.')),
     cfg.StrOpt('onready',
-               help=_('onready allows you to send a notification when the'
-                      ' heat processes are ready to serve.  This is either a'
-                      ' module with the notify() method or a shell command. '
-                      ' To enable notifications with systemd, one may use'
-                      ' the \'systemd-notify --ready\' shell command or'
-                      ' the \'heat.common.systemd\' notification module.'))]
+               help=_('Deprecated.'))]
 
 rpc_opts = [
     cfg.StrOpt('host',
@@ -155,7 +170,9 @@ auth_password_opts = [
                 help=_('Allowed keystone endpoints for auth_uri when '
                        'multi_cloud is enabled. At least one endpoint needs '
                        'to be specified.'))]
-clients_opts = [
+
+# these options define baseline defaults that apply to all clients
+default_clients_opts = [
     cfg.StrOpt('endpoint_type',
                default='publicURL',
                help=_(
@@ -173,23 +190,35 @@ clients_opts = [
                 help=_("If set, then the server's certificate will not "
                        "be verified."))]
 
+# these options can be defined for each client
+# they must not specify defaults, since any options not defined in a client
+# specific group is looked up on the generic group above
+clients_opts = [
+    cfg.StrOpt('endpoint_type',
+               help=_(
+                   'Type of endpoint in Identity service catalog to use '
+                   'for communication with the OpenStack service.')),
+    cfg.StrOpt('ca_file',
+               help=_('Optional CA cert file to use in SSL connections.')),
+    cfg.StrOpt('cert_file',
+               help=_('Optional PEM-formatted certificate chain file.')),
+    cfg.StrOpt('key_file',
+               help=_('Optional PEM-formatted file that contains the '
+                      'private key.')),
+    cfg.BoolOpt('insecure',
+                help=_("If set, then the server's certificate will not "
+                       "be verified."))]
 
-def register_clients_opts():
-    cfg.CONF.register_opts(clients_opts, group='clients')
-    for client in ('nova', 'swift', 'neutron', 'cinder',
-                   'ceilometer', 'keystone', 'heat', 'trove'):
-        client_specific_group = 'clients_' + client
-        # register opts copy and put it to globals in order to
-        # generate_sample.sh to work
-        opts_copy = copy.deepcopy(clients_opts)
-        if client == 'heat':
-            opts_copy.append(
-                cfg.StrOpt('url',
-                           help=_('Optional heat url in format like'
-                                  ' http://0.0.0.0:8004/v1/%(tenant_id)s.')))
-        globals()[client_specific_group + '_opts'] = opts_copy
-        cfg.CONF.register_opts(opts_copy, group=client_specific_group)
+heat_client_opts = [
+    cfg.StrOpt('url',
+               default='',
+               help=_('Optional heat url in format like'
+                      ' http://0.0.0.0:8004/v1/%(tenant_id)s.'))]
 
+client_http_log_debug_opts = [
+    cfg.BoolOpt('http_log_debug',
+                default=False,
+                help=_("Allow client's debug log output."))]
 
 revision_group = cfg.OptGroup('revision')
 revision_opts = [
@@ -200,32 +229,32 @@ revision_opts = [
                       'separately, you can move this section to a different '
                       'file and add it as another config option.'))]
 
-cfg.CONF.register_opts(engine_opts)
-cfg.CONF.register_opts(service_opts)
-cfg.CONF.register_opts(rpc_opts)
-rpc.set_defaults(control_exchange='heat')
+
+def list_opts():
+    yield None, rpc_opts
+    yield None, engine_opts
+    yield None, service_opts
+    yield paste_deploy_group.name, paste_deploy_opts
+    yield auth_password_group.name, auth_password_opts
+    yield revision_group.name, revision_opts
+    yield 'clients', default_clients_opts
+
+    for client in ('nova', 'swift', 'neutron', 'cinder',
+                   'ceilometer', 'keystone', 'heat', 'glance', 'trove'):
+        client_specific_group = 'clients_' + client
+        yield client_specific_group, clients_opts
+
+    yield 'clients_heat', heat_client_opts
+    yield 'clients_nova', client_http_log_debug_opts
+    yield 'clients_cinder', client_http_log_debug_opts
+
+
 cfg.CONF.register_group(paste_deploy_group)
-cfg.CONF.register_opts(paste_deploy_opts, group=paste_deploy_group)
 cfg.CONF.register_group(auth_password_group)
-cfg.CONF.register_opts(auth_password_opts, group=auth_password_group)
 cfg.CONF.register_group(revision_group)
-cfg.CONF.register_opts(revision_opts, group=revision_group)
-register_clients_opts()
 
-# A bit of history:
-# This was added initially by jianingy, then it got added
-# to oslo by Luis. Then it was receintly removed from the
-# default list again.
-# I am not sure we can (or should) rely on oslo to keep
-# our exceptions class in the defaults list.
-allowed_rpc_exception_modules = cfg.CONF.allowed_rpc_exception_modules
-allowed_rpc_exception_modules.append('heat.common.exception')
-cfg.CONF.set_default(name='allowed_rpc_exception_modules',
-                     default=allowed_rpc_exception_modules)
-
-if cfg.CONF.instance_user:
-    logger.warn(_('The "instance_user" option in heat.conf is deprecated and '
-                  'will be removed in the Juno release.'))
+for group, opts in list_opts():
+    cfg.CONF.register_opts(opts, group=group)
 
 
 def _get_deployment_flavor():
