@@ -248,13 +248,16 @@ class KeystoneClientV3(object):
         return opts
 
     def _get_client_option(self, option):
-        try:
-            cfg.CONF.import_opt(option, 'heat.common.config',
-                                group='clients_keystone')
-            return getattr(cfg.CONF.clients_keystone, option)
-        except (cfg.NoSuchGroupError, cfg.NoSuchOptError):
-            cfg.CONF.import_opt(option, 'heat.common.config', group='clients')
-            return getattr(cfg.CONF.clients, option)
+        # look for the option in the [clients_keystone] section
+        # unknown options raise cfg.NoSuchOptError
+        cfg.CONF.import_opt(option, 'heat.common.config',
+                            group='clients_keystone')
+        v = getattr(cfg.CONF.clients_keystone, option)
+        if v is not None:
+            return v
+        # look for the option in the generic [clients] section
+        cfg.CONF.import_opt(option, 'heat.common.config', group='clients')
+        return getattr(cfg.CONF.clients, option)
 
     def create_trust_context(self):
         """Create a trust using the trustor identity in the current context.
@@ -275,7 +278,11 @@ class KeystoneClientV3(object):
         trustee_user_id = self.admin_client.auth_ref.user_id
         trustor_user_id = self.client.auth_ref.user_id
         trustor_project_id = self.client.auth_ref.project_id
-        roles = cfg.CONF.trusts_delegated_roles
+        # inherit the roles of the trustor, unless set trusts_delegated_roles
+        if cfg.CONF.trusts_delegated_roles:
+            roles = cfg.CONF.trusts_delegated_roles
+        else:
+            roles = self.context.roles
         try:
             trust = self.client.trusts.create(trustor_user=trustor_user_id,
                                               trustee_user=trustee_user_id,
@@ -308,6 +315,21 @@ class KeystoneClientV3(object):
         #get the last 64 characters of the username
         return username[-64:]
 
+    def _get_domain_id_from_name(self, domain_name):
+        domains = self.domain_admin_client.domains.list(name=domain_name)
+        if len(domains) == 1:
+            return domains[0].id
+        elif len(domains) == 0:
+            LOG.error(_LE("Can't find domain id for %(domain)s!"), {
+                      'domain': domain_name})
+            raise exception.Error(_("Failed to find domain %s")
+                                  % domain_name)
+        else:
+            LOG.error(_LE("Unexpected response looking for %(domain)s!"), {
+                      'domain': domain_name})
+            raise exception.Error(_("Unexpected response looking for "
+                                    "domain %s") % domain_name)
+
     def create_stack_user(self, username, password=''):
         """Create a user defined as part of a stack.
 
@@ -415,7 +437,7 @@ class KeystoneClientV3(object):
             # Create user
             user = self.domain_admin_client.users.create(
                 name=self._get_username(username), password=password,
-                default_project=project_id, domain=self.stack_domain)
+                default_project=project_id, domain=self.stack_domain_id)
             # Add to stack user role
             LOG.debug("Adding user %(user)s to role %(role)s" % {
                       'user': user.id, 'role': role_id})
@@ -437,9 +459,8 @@ class KeystoneClientV3(object):
             if self._stack_domain_is_id:
                 self._stack_domain_id = self.stack_domain
             else:
-                domain = self.domain_admin_client.domains.get(
-                    self.stack_domain)
-                self._stack_domain_id = domain.id
+                domain_id = self._get_domain_id_from_name(self.stack_domain)
+                self._stack_domain_id = domain_id
         return self._stack_domain_id
 
     def _check_stack_domain_user(self, user_id, project_id, action):
@@ -485,7 +506,7 @@ class KeystoneClientV3(object):
         desc = "Heat stack user project"
         domain_project = self.domain_admin_client.projects.create(
             name=project_name,
-            domain=self.stack_domain,
+            domain=self.stack_domain_id,
             description=desc)
         return domain_project.id
 
