@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -12,15 +11,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo.config import cfg
+
 from heat.common import exception
 from heat.engine import constraints
 from heat.engine import properties
-from heat.engine.properties import Properties
 from heat.engine import resource
+from heat.engine import support
 from heat.engine import watchrule
-from heat.openstack.common import log as logging
-
-logger = logging.getLogger(__name__)
 
 
 class CloudWatchAlarm(resource.Resource):
@@ -134,7 +132,10 @@ class CloudWatchAlarm(resource.Resource):
 
     strict_dependency = False
 
-    update_allowed_keys = ('Properties',)
+    support_status = support.SupportStatus(
+        status=support.DEPRECATED,
+        message=_('OS::Heat::CWLiteAlarm is deprecated, '
+                  'use OS::Ceilometer::Alarm instead.'))
 
     def handle_create(self):
         wr = watchrule.WatchRule(context=self.context,
@@ -147,11 +148,8 @@ class CloudWatchAlarm(resource.Resource):
         # If Properties has changed, update self.properties, so we
         # get the new values during any subsequent adjustment
         if prop_diff:
-            self.properties = Properties(self.properties_schema,
-                                         json_snippet.get('Properties', {}),
-                                         self.stack.resolve_runtime_data,
-                                         self.name,
-                                         self.context)
+            self.properties = json_snippet.properties(self.properties_schema,
+                                                      self.context)
             loader = watchrule.WatchRule.load
             wr = loader(self.context,
                         watch_name=self.physical_resource_name())
@@ -179,13 +177,17 @@ class CloudWatchAlarm(resource.Resource):
         wr.state_set(wr.NODATA)
 
     def FnGetRefId(self):
-        return unicode(self.physical_resource_name())
+        return self.physical_resource_name_or_FnGetRefId()
 
     def physical_resource_name(self):
         return '%s-%s' % (self.stack.name, self.name)
 
 
 def resource_mapping():
-    return {
-        'OS::Heat::CWLiteAlarm': CloudWatchAlarm,
-    }
+    cfg.CONF.import_opt('enable_cloud_watch_lite', 'heat.common.config')
+    if cfg.CONF.enable_cloud_watch_lite:
+        return {
+            'OS::Heat::CWLiteAlarm': CloudWatchAlarm,
+        }
+    else:
+        return {}
