@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -12,20 +11,21 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutronclient.common.exceptions import NeutronClientException
+import warnings
 
 from heat.common import exception
-from heat.engine.properties import Properties
 from heat.engine import resource
 from heat.engine import scheduler
 from heat.openstack.common import log as logging
 from heat.openstack.common import uuidutils
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class NeutronResource(resource.Resource):
 
+    default_client_name = 'neutron'
+
     def validate(self):
         '''
         Validate any of the provided params
@@ -51,6 +51,21 @@ class NeutronResource(resource.Resource):
             for k in banned_keys.intersection(vs.keys()):
                 return '%s not allowed in value_specs' % k
 
+    @staticmethod
+    def _validate_depr_property_required(properties, prop_key, depr_prop_key):
+            prop_value = properties.get(prop_key)
+            depr_prop_value = properties.get(depr_prop_key)
+
+            if prop_value and depr_prop_value:
+                raise exception.ResourcePropertyConflict(prop_key,
+                                                         depr_prop_key)
+            if not prop_value and not depr_prop_value:
+                msg = _('Either %(prop_key)s or %(depr_prop_key)s'
+                        ' should be specified.'
+                        ) % {'prop_key': prop_key,
+                             'depr_prop_key': depr_prop_key}
+                raise exception.StackValidationFailed(message=msg)
+
     @staticmethod
     def prepare_properties(properties, name):
         '''
@@ -71,7 +86,7 @@ class NeutronResource(resource.Resource):
 
         return props
 
-    def prepare_update_properties(self, json_snippet):
+    def prepare_update_properties(self, definition):
         '''
         Prepares the property values so that they can be passed directly to
         the Neutron update call.
@@ -79,11 +94,7 @@ class NeutronResource(resource.Resource):
         Removes any properties which are not update_allowed, then processes
         as for prepare_properties.
         '''
-        p = Properties(self.properties_schema,
-                       json_snippet.get('Properties', {}),
-                       self._resolve_runtime_data,
-                       self.name,
-                       self.context)
+        p = definition.properties(self.properties_schema, self.context)
         update_props = dict((k, v) for k, v in p.items()
                             if p.props.get(k).schema.update_allowed)
 
@@ -92,64 +103,56 @@ class NeutronResource(resource.Resource):
             self.physical_resource_name())
         return props
 
-    @staticmethod
-    def handle_get_attributes(name, key, attributes):
-        '''
-        Support method for responding to FnGetAtt
-        '''
-        if key == 'show':
-            return attributes
-
-        if key in attributes.keys():
-            return attributes[key]
-
-        raise exception.InvalidTemplateAttribute(resource=name, key=key)
-
     @staticmethod
     def is_built(attributes):
-        if attributes['status'] == 'BUILD':
+        status = attributes['status']
+        if status == 'BUILD':
             return False
-        if attributes['status'] in ('ACTIVE', 'DOWN'):
+        if status in ('ACTIVE', 'DOWN'):
             return True
+        elif status == 'ERROR':
+            raise resource.ResourceInError(
+                resource_status=status)
         else:
-            raise exception.Error(_('neutron reported unexpected '
-                                    'resource[%(name)s] status[%(status)s]') %
-                                  {'name': attributes['name'],
-                                   'status': attributes['status']})
+            raise resource.ResourceUnknownStatus(
+                resource_status=status,
+                result=_('Resource is not built'))
 
     def _resolve_attribute(self, name):
         try:
             attributes = self._show_resource()
-        except NeutronClientException as ex:
-            logger.warn(_("failed to fetch resource attributes: %s") %
-                        str(ex))
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
             return None
-        return self.handle_get_attributes(self.name, name, attributes)
+        if name == 'show':
+            return attributes
+
+        return attributes[name]
 
     def _confirm_delete(self):
         while True:
             try:
                 yield
                 self._show_resource()
-            except NeutronClientException as ex:
-                self._handle_not_found_exception(ex)
+            except Exception as ex:
+                self.client_plugin().ignore_not_found(ex)
                 return
 
-    def _handle_not_found_exception(self, ex):
-        if ex.status_code != 404:
-            raise ex
-
     def FnGetRefId(self):
         return unicode(self.resource_id)
 
     @staticmethod
-    def get_secgroup_uuids(security_groups, client):
+    def get_secgroup_uuids(security_groups, client, tenant_id):
         '''
         Returns a list of security group UUIDs.
         Args:
             security_groups: List of security group names or UUIDs
             client: reference to neutronclient
+            tenant_id: the tenant id to match the security_groups
         '''
+        warnings.warn('neutron.NeutronResource.get_secgroup_uuids is '
+                      'deprecated. Use '
+                      'self.client_plugin("neutron").get_secgroup_uuids')
         seclist = []
         all_groups = None
         for sg in security_groups:
@@ -159,12 +162,22 @@ class NeutronResource(resource.Resource):
                 if not all_groups:
                     response = client.list_security_groups()
                     all_groups = response['security_groups']
-                groups = [g['id'] for g in all_groups if g['name'] == sg]
+                same_name_groups = [g for g in all_groups if g['name'] == sg]
+                groups = [g['id'] for g in same_name_groups]
                 if len(groups) == 0:
                     raise exception.PhysicalResourceNotFound(resource_id=sg)
-                if len(groups) > 1:
-                    raise exception.PhysicalResourceNameAmbiguity(name=sg)
-                seclist.append(groups[0])
+                elif len(groups) == 1:
+                    seclist.append(groups[0])
+                else:
+                    # for admin roles, can get the other users'
+                    # securityGroups, so we should match the tenant_id with
+                    # the groups, and return the own one
+                    own_groups = [g['id'] for g in same_name_groups
+                                  if g['tenant_id'] == tenant_id]
+                    if len(own_groups) == 1:
+                        seclist.append(own_groups[0])
+                    else:
+                        raise exception.PhysicalResourceNameAmbiguity(name=sg)
         return seclist
 
     def _delete_task(self):
