@@ -1,3 +1,4 @@
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -10,19 +11,26 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from heat.engine import clients
+from oslo.utils import excutils
+
+from heat.common.i18n import _
+from heat.engine import attributes
 from heat.engine import properties
 from heat.engine import resource
-from heat.openstack.common import excutils
-from heat.openstack.common.gettextutils import _
 from heat.openstack.common import log as logging
 
-logger = logging.getLogger(__name__)
+LOG = logging.getLogger(__name__)
 
 
 class NovaFloatingIp(resource.Resource):
     PROPERTIES = (POOL,) = ('pool',)
 
+    ATTRIBUTES = (
+        POOL_ATTR, IP,
+    ) = (
+        'pool', 'ip',
+    )
+
     properties_schema = {
         POOL: properties.Schema(
             properties.Schema.STRING,
@@ -32,10 +40,16 @@ class NovaFloatingIp(resource.Resource):
     }
 
     attributes_schema = {
-        'pool': _('Pool from which floating IP is allocated.'),
-        'ip': _('Allocated floating IP address.')
+        POOL_ATTR: attributes.Schema(
+            _('Pool from which floating IP is allocated.')
+        ),
+        IP: attributes.Schema(
+            _('Allocated floating IP address.')
+        ),
     }
 
+    default_client_name = 'nova'
+
     def __init__(self, name, json_snippet, stack):
         super(NovaFloatingIp, self).__init__(name, json_snippet, stack)
         self._floating_ip = None
@@ -50,12 +64,14 @@ class NovaFloatingIp(resource.Resource):
         try:
             pool = self.properties.get(self.POOL)
             floating_ip = self.nova().floating_ips.create(pool=pool)
-        except clients.novaclient.exceptions.NotFound:
+        except Exception as e:
             with excutils.save_and_reraise_exception():
-                if pool is None:
-                    msg = _('Could not allocate floating IP. Probably there '
-                            'is no default floating IP pool is configured.')
-                    logger.error(msg)
+                if self.client_plugin().is_not_found(e):
+                    if pool is None:
+                        msg = _('Could not allocate floating IP. Probably '
+                                'there is no default floating IP pool is '
+                                'configured.')
+                        LOG.error(msg)
 
         self.resource_id_set(floating_ip.id)
         self._floating_ip = floating_ip
@@ -64,14 +80,14 @@ class NovaFloatingIp(resource.Resource):
         if self.resource_id is not None:
             try:
                 self.nova().floating_ips.delete(self.resource_id)
-            except clients.novaclient.exceptions.NotFound:
-                pass
+            except Exception as e:
+                self.client_plugin().ignore_not_found(e)
 
     def _resolve_attribute(self, key):
         floating_ip = self._get_resource()
         attributes = {
-            'pool': getattr(floating_ip, 'pool', None),
-            'ip': floating_ip.ip
+            self.POOL_ATTR: getattr(floating_ip, self.POOL_ATTR, None),
+            self.IP: floating_ip.ip
         }
         return unicode(attributes[key])
 
@@ -87,17 +103,19 @@ class NovaFloatingIpAssociation(resource.Resource):
         SERVER: properties.Schema(
             properties.Schema.STRING,
             _('Server to assign floating IP to.'),
-            required=True
+            required=True,
+            update_allowed=True
         ),
         FLOATING_IP: properties.Schema(
             properties.Schema.STRING,
             _('ID of the floating IP to assign to the server.'),
-            required=True
+            required=True,
+            update_allowed=True
         ),
     }
 
     def FnGetRefId(self):
-        return unicode(self.physical_resource_name())
+        return self.physical_resource_name_or_FnGetRefId()
 
     def handle_create(self):
         server = self.nova().servers.get(self.properties[self.SERVER])
@@ -107,14 +125,37 @@ class NovaFloatingIpAssociation(resource.Resource):
         self.resource_id_set('%s-%s' % (fl_ip.id, fl_ip.ip))
 
     def handle_delete(self):
+        if self.resource_id is None:
+            return
+
         try:
             server = self.nova().servers.get(self.properties[self.SERVER])
             if server:
                 fl_ip = self.nova().floating_ips.\
                     get(self.properties[self.FLOATING_IP])
                 self.nova().servers.remove_floating_ip(server, fl_ip.ip)
-        except clients.novaclient.exceptions.NotFound:
-            pass
+        except Exception as e:
+            self.client_plugin().ignore_not_found(e)
+
+    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
+        if prop_diff:
+            # If floating_ip in prop_diff, we need to remove the old floating
+            # ip from the old server, and then to add the new floating ip
+            # to the old/new(if the server_id is changed) server.
+            # If prop_diff only has the server_id, no need to remove the
+            # floating ip from the old server, nova does this automatically
+            # when calling add_floating_ip().
+            if self.FLOATING_IP in prop_diff:
+                self.handle_delete()
+            server_id = (prop_diff.get(self.SERVER) or
+                         self.properties[self.SERVER])
+            fl_ip_id = (prop_diff.get(self.FLOATING_IP) or
+                        self.properties[self.FLOATING_IP])
+            server = self.nova().servers.get(server_id)
+            fl_ip = self.nova().floating_ips.get(fl_ip_id)
+
+            self.nova().servers.add_floating_ip(server, fl_ip.ip)
+            self.resource_id_set('%s-%s' % (fl_ip.id, fl_ip.ip))
 
 
 def resource_mapping():
