@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -17,12 +16,9 @@ from requests import exceptions
 from heat.common import exception
 from heat.common import template_format
 from heat.common import urlfetch
+from heat.engine import attributes
 from heat.engine import properties
-from heat.engine.properties import Properties
 from heat.engine import stack_resource
-from heat.openstack.common import log as logging
-
-logger = logging.getLogger(__name__)
 
 
 class NestedStack(stack_resource.StackResource):
@@ -57,8 +53,6 @@ class NestedStack(stack_resource.StackResource):
         ),
     }
 
-    update_allowed_keys = ('Properties',)
-
     def child_template(self):
         try:
             template_data = urlfetch.get(self.properties[self.TEMPLATE_URL])
@@ -89,22 +83,20 @@ class NestedStack(stack_resource.StackResource):
     def handle_delete(self):
         return self.delete_nested()
 
-    def FnGetAtt(self, key):
+    def FnGetAtt(self, key, *path):
         if key and not key.startswith('Outputs.'):
             raise exception.InvalidTemplateAttribute(resource=self.name,
                                                      key=key)
-        return self.get_output(key.partition('.')[-1])
+        attribute = self.get_output(key.partition('.')[-1])
+        return attributes.select_from_attribute(attribute, path)
 
     def FnGetRefId(self):
         return self.nested().identifier().arn()
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
         # Nested stack template may be changed even if the prop_diff is empty.
-        self.properties = Properties(self.properties_schema,
-                                     json_snippet.get('Properties', {}),
-                                     self.stack.resolve_runtime_data,
-                                     self.name,
-                                     self.context)
+        self.properties = json_snippet.properties(self.properties_schema,
+                                                  self.context)
 
         try:
             template_data = urlfetch.get(self.properties[self.TEMPLATE_URL])
