@@ -1,4 +1,4 @@
-
+#
 # Copyright 2012, Red Hat, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -18,14 +18,13 @@ Unit Tests for heat.rpc.client
 """
 
 
+import copy
 import mock
-from oslo.config import cfg
 import stubout
 import testtools
 
 from heat.common import identifier
-from heat.openstack.common import rpc
-from heat.rpc import api as rpc_api
+from heat.common import messaging
 from heat.rpc import client as rpc_client
 from heat.tests import utils
 
@@ -33,46 +32,40 @@ from heat.tests import utils
 class EngineRpcAPITestCase(testtools.TestCase):
 
     def setUp(self):
+        messaging.setup("fake://", optional=True)
+        self.addCleanup(messaging.cleanup)
         self.context = utils.dummy_context()
-        cfg.CONF.set_default('rpc_backend',
-                             'heat.openstack.common.rpc.impl_fake')
-        cfg.CONF.set_default('verbose', True)
-        cfg.CONF.set_default('host', 'host')
 
         self.stubs = stubout.StubOutForTesting()
         self.identity = dict(identifier.HeatIdentifier('engine_test_tenant',
                                                        '6',
                                                        'wordpress'))
+        self.rpcapi = rpc_client.EngineClient()
         super(EngineRpcAPITestCase, self).setUp()
 
     def _test_engine_api(self, method, rpc_method, **kwargs):
         ctxt = utils.dummy_context()
-        if 'rpcapi_class' in kwargs:
-            rpcapi_class = kwargs['rpcapi_class']
-            del kwargs['rpcapi_class']
-        else:
-            rpcapi_class = rpc_client.EngineClient
-        rpcapi = rpcapi_class()
         expected_retval = 'foo' if method == 'call' else None
 
-        expected_version = kwargs.pop('version', rpcapi.BASE_RPC_API_VERSION)
-        expected_msg = rpcapi.make_msg(method, **kwargs)
+        kwargs.pop('version', None)
 
-        expected_msg['version'] = expected_version
-        expected_topic = rpc_api.ENGINE_TOPIC
+        if 'expected_message' in kwargs:
+            expected_message = kwargs['expected_message']
+            del kwargs['expected_message']
+        else:
+            expected_message = self.rpcapi.make_msg(method, **kwargs)
 
         cast_and_call = ['delete_stack']
         if rpc_method == 'call' and method in cast_and_call:
             kwargs['cast'] = False
 
-        with mock.patch.object(rpc, rpc_method) as mock_rpc_method:
+        with mock.patch.object(self.rpcapi, rpc_method) as mock_rpc_method:
             mock_rpc_method.return_value = expected_retval
 
-            retval = getattr(rpcapi, method)(ctxt, **kwargs)
+            retval = getattr(self.rpcapi, method)(ctxt, **kwargs)
 
             self.assertEqual(expected_retval, retval)
-            expected_args = [ctxt, expected_topic, expected_msg,
-                             mock.ANY]
+            expected_args = [ctxt, expected_message, mock.ANY]
             actual_args, _ = mock_rpc_method.call_args
             for expected_arg, actual_arg in zip(expected_args,
                                                 actual_args):
@@ -89,6 +82,8 @@ class EngineRpcAPITestCase(testtools.TestCase):
             'sort_dir': mock.ANY,
             'filters': mock.ANY,
             'tenant_safe': mock.ANY,
+            'show_deleted': mock.ANY,
+            'show_nested': mock.ANY,
         }
         self._test_engine_api('list_stacks', 'call', **default_args)
 
@@ -96,6 +91,8 @@ class EngineRpcAPITestCase(testtools.TestCase):
         default_args = {
             'filters': mock.ANY,
             'tenant_safe': mock.ANY,
+            'show_deleted': mock.ANY,
+            'show_nested': mock.ANY,
         }
         self._test_engine_api('count_stacks', 'call', **default_args)
 
@@ -114,11 +111,16 @@ class EngineRpcAPITestCase(testtools.TestCase):
                               args={'timeout_mins': u'30'})
 
     def test_create_stack(self):
-        self._test_engine_api('create_stack', 'call', stack_name='wordpress',
-                              template={u'Foo': u'bar'},
-                              params={u'InstanceType': u'm1.xlarge'},
-                              files={u'a_file': u'the contents'},
-                              args={'timeout_mins': u'30'})
+        kwargs = dict(stack_name='wordpress',
+                      template={u'Foo': u'bar'},
+                      params={u'InstanceType': u'm1.xlarge'},
+                      files={u'a_file': u'the contents'},
+                      args={'timeout_mins': u'30'})
+        call_kwargs = copy.deepcopy(kwargs)
+        call_kwargs['owner_id'] = None
+        expected_message = self.rpcapi.make_msg('create_stack', **call_kwargs)
+        kwargs['expected_message'] = expected_message
+        self._test_engine_api('create_stack', 'call', **kwargs)
 
     def test_update_stack(self):
         self._test_engine_api('update_stack', 'call',
@@ -156,8 +158,13 @@ class EngineRpcAPITestCase(testtools.TestCase):
         self._test_engine_api('generate_template', 'call', type_name="TYPE")
 
     def test_list_events(self):
-        self._test_engine_api('list_events', 'call',
-                              stack_identity=self.identity)
+        kwargs = {'stack_identity': self.identity,
+                  'limit': None,
+                  'marker': None,
+                  'sort_keys': None,
+                  'sort_dir': None,
+                  'filters': None}
+        self._test_engine_api('list_events', 'call', **kwargs)
 
     def test_describe_stack_resource(self):
         self._test_engine_api('describe_stack_resource', 'call',
@@ -175,7 +182,8 @@ class EngineRpcAPITestCase(testtools.TestCase):
 
     def test_list_stack_resources(self):
         self._test_engine_api('list_stack_resources', 'call',
-                              stack_identity=self.identity)
+                              stack_identity=self.identity,
+                              nested_depth=0)
 
     def test_stack_suspend(self):
         self._test_engine_api('stack_suspend', 'call',
@@ -185,6 +193,10 @@ class EngineRpcAPITestCase(testtools.TestCase):
         self._test_engine_api('stack_resume', 'call',
                               stack_identity=self.identity)
 
+    def test_stack_cancel_update(self):
+        self._test_engine_api('stack_cancel_update', 'call',
+                              stack_identity=self.identity)
+
     def test_metadata_update(self):
         self._test_engine_api('metadata_update', 'call',
                               stack_identity=self.identity,
@@ -268,3 +280,20 @@ class EngineRpcAPITestCase(testtools.TestCase):
         deployment_id = '86729f02-4648-44d8-af44-d0ec65b6abc9'
         self._test_engine_api('delete_software_deployment', 'call',
                               deployment_id=deployment_id)
+
+    def test_show_snapshot(self):
+        snapshot_id = '86729f02-4648-44d8-af44-d0ec65b6abc9'
+        self._test_engine_api('show_snapshot', 'call',
+                              stack_identity=self.identity,
+                              snapshot_id=snapshot_id)
+
+    def test_stack_snapshot(self):
+        self._test_engine_api(
+            'stack_snapshot', 'call', stack_identity=self.identity,
+            name='snap1')
+
+    def test_delete_snapshot(self):
+        snapshot_id = '86729f02-4648-44d8-af44-d0ec65b6abc9'
+        self._test_engine_api('delete_snapshot', 'call',
+                              stack_identity=self.identity,
+                              snapshot_id=snapshot_id)
