@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -12,20 +12,15 @@
 #    under the License.
 
 
-from testtools import skipIf
+import swiftclient.client as sc
 
 from heat.common import exception
 from heat.common import template_format
-from heat.engine import clients
-from heat.engine import resource
 from heat.engine.resources import s3
 from heat.engine import scheduler
-from heat.openstack.common.importutils import try_import
 from heat.tests.common import HeatTestCase
-from heat.tests import fakes
 from heat.tests import utils
 
-swiftclient = try_import('swiftclient.client')
 
 swift_template = '''
 {
@@ -68,37 +63,33 @@ swift_template = '''
 
 
 class s3Test(HeatTestCase):
-    @skipIf(swiftclient is None, 'unable to import swiftclient')
     def setUp(self):
         super(s3Test, self).setUp()
-        self.m.CreateMock(swiftclient.Connection)
-        self.m.StubOutWithMock(swiftclient.Connection, 'put_container')
-        self.m.StubOutWithMock(swiftclient.Connection, 'delete_container')
-        self.m.StubOutWithMock(swiftclient.Connection, 'get_auth')
-        self.m.StubOutWithMock(clients.OpenStackClients, 'keystone')
-
-        utils.setup_dummy_db()
+        self.m.CreateMock(sc.Connection)
+        self.m.StubOutWithMock(sc.Connection, 'put_container')
+        self.m.StubOutWithMock(sc.Connection, 'delete_container')
+        self.m.StubOutWithMock(sc.Connection, 'get_auth')
+        self.stub_keystoneclient()
 
     def create_resource(self, t, stack, resource_name):
+        resource_defns = stack.t.resource_definitions(stack)
         rsrc = s3.S3Bucket('test_resource',
-                           t['Resources'][resource_name],
+                           resource_defns[resource_name],
                            stack)
         scheduler.TaskRunner(rsrc.create)()
         self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
         return rsrc
 
     def test_attributes(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name,
             {'X-Container-Write': 'test_tenant:test_username',
              'X-Container-Read': 'test_tenant:test_username'}
         ).AndReturn(None)
-        swiftclient.Connection.get_auth().MultipleTimes().AndReturn(
+        sc.Connection.get_auth().MultipleTimes().AndReturn(
             ('http://server.test:8080/v_2', None))
-        swiftclient.Connection.delete_container(container_name).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndReturn(None)
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -116,21 +107,16 @@ class s3Test(HeatTestCase):
         self.assertRaises(exception.InvalidTemplateAttribute,
                           rsrc.FnGetAtt, 'Foo')
 
-        self.assertRaises(resource.UpdateReplace,
-                          rsrc.handle_update, {}, {}, {})
-
         scheduler.TaskRunner(rsrc.delete)()
         self.m.VerifyAll()
 
     def test_public_read(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             utils.PhysName('test_stack', 'test_resource'),
             {'X-Container-Write': 'test_tenant:test_username',
              'X-Container-Read': '.r:*'}).AndReturn(None)
-        swiftclient.Connection.delete_container(
+        sc.Connection.delete_container(
             container_name).AndReturn(None)
 
         self.m.ReplayAll()
@@ -143,16 +129,14 @@ class s3Test(HeatTestCase):
         self.m.VerifyAll()
 
     def test_tags(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             utils.PhysName('test_stack', 'test_resource'),
             {'X-Container-Write': 'test_tenant:test_username',
              'X-Container-Read': 'test_tenant:test_username',
              'X-Container-Meta-S3-Tag-greeting': 'hello',
              'X-Container-Meta-S3-Tag-location': 'here'}).AndReturn(None)
-        swiftclient.Connection.delete_container(
+        sc.Connection.delete_container(
             container_name).AndReturn(None)
 
         self.m.ReplayAll()
@@ -163,14 +147,12 @@ class s3Test(HeatTestCase):
         self.m.VerifyAll()
 
     def test_public_read_write(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name,
             {'X-Container-Write': '.r:*',
              'X-Container-Read': '.r:*'}).AndReturn(None)
-        swiftclient.Connection.delete_container(
+        sc.Connection.delete_container(
             container_name).AndReturn(None)
 
         self.m.ReplayAll()
@@ -183,14 +165,12 @@ class s3Test(HeatTestCase):
         self.m.VerifyAll()
 
     def test_authenticated_read(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name,
             {'X-Container-Write': 'test_tenant:test_username',
              'X-Container-Read': 'test_tenant'}).AndReturn(None)
-        swiftclient.Connection.delete_container(container_name).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndReturn(None)
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -202,16 +182,14 @@ class s3Test(HeatTestCase):
         self.m.VerifyAll()
 
     def test_website(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name,
             {'X-Container-Meta-Web-Error': 'error.html',
              'X-Container-Meta-Web-Index': 'index.html',
              'X-Container-Write': 'test_tenant:test_username',
              'X-Container-Read': '.r:*'}).AndReturn(None)
-        swiftclient.Connection.delete_container(container_name).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndReturn(None)
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -221,15 +199,31 @@ class s3Test(HeatTestCase):
         self.m.VerifyAll()
 
     def test_delete_exception(self):
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         container_name = utils.PhysName('test_stack', 'test_resource')
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             container_name,
             {'X-Container-Write': 'test_tenant:test_username',
              'X-Container-Read': 'test_tenant:test_username'}).AndReturn(None)
-        swiftclient.Connection.delete_container(container_name).AndRaise(
-            swiftclient.ClientException('Test delete failure'))
+        sc.Connection.delete_container(container_name).AndRaise(
+            sc.ClientException('Test delete failure'))
+
+        self.m.ReplayAll()
+        t = template_format.parse(swift_template)
+        stack = utils.parse_stack(t)
+        rsrc = self.create_resource(t, stack, 'S3Bucket')
+        self.assertRaises(exception.ResourceFailure,
+                          scheduler.TaskRunner(rsrc.delete))
+
+        self.m.VerifyAll()
+
+    def test_delete_not_found(self):
+        container_name = utils.PhysName('test_stack', 'test_resource')
+        sc.Connection.put_container(
+            container_name,
+            {'X-Container-Write': 'test_tenant:test_username',
+             'X-Container-Read': 'test_tenant:test_username'}).AndReturn(None)
+        sc.Connection.delete_container(container_name).AndRaise(
+            sc.ClientException('Its gone', http_status=404))
 
         self.m.ReplayAll()
         t = template_format.parse(swift_template)
@@ -240,11 +234,8 @@ class s3Test(HeatTestCase):
         self.m.VerifyAll()
 
     def test_delete_retain(self):
-
-        clients.OpenStackClients.keystone().AndReturn(
-            fakes.FakeKeystoneClient())
         # first run, with retain policy
-        swiftclient.Connection.put_container(
+        sc.Connection.put_container(
             utils.PhysName('test_stack', 'test_resource'),
             {'X-Container-Write': 'test_tenant:test_username',
              'X-Container-Read': 'test_tenant:test_username'}).AndReturn(None)
